
library("reshape2")
library("dplyr")
library("rstan")
library("polspline")
library("MultinomialCI")

options(mc.cores = parallel::detectCores())
rstan_options(auto_write = TRUE)
windowsFonts(Arial=windowsFont("TT Arial"))

#####################################################
################ Data preprocessing #################
#####################################################

d <- read.csv("all_data.csv")

## remove participants: multiple or missing IP entries, no serious participation and shady response times  ##

ips <- d %>% group_by(ID) %>%
  summarise(ip = first(IP),
            full_t = max(time_elapsed)/1000/60,
            mean_rt = mean(rt)/1000,
            useful = first(useful),
            complete = first(complete))

# list multiple IP entries          
tab_ips <- table(ips$ip) #count number of entries per IP
mult_ip <- names(tab_ips[tab_ips > 1]) # find the IPs with multiple entries
exclude_ip <- ips %>% filter((ip %in% mult_ip)) %>% select(ID) %>% unlist() %>% as.character # list their IDs

# list missing IP
exclude_missing_ip <- as.character(ips$ID[is.na(ips$ip)]) 

# list shady reaction times
exclude_rt <- as.character(ips$ID[ips$mean_rt < 2]) # cut off: mean rt for trials < 2 seconds

# list useles data
exclude_useles <- as.character(ips$ID[ips$useful == FALSE]) 

# exclude the participants listed above
exclude <- c(exclude_ip,exclude_missing_ip, exclude_rt, exclude_useles) 
dr  <-  droplevels(d[!(d$ID %in% exclude),])

# nrow(d)
# nrow(dr)
# length(levels(d$ID))
# length(levels(dr$ID))

## wide format
dw <- dcast(dr, ID ~ Believability + Validity + button_pressed, length)

#####################################################
####### Plot responeses with multinomial CI #########
#####################################################


freq <- dcast(dr,  Believability ~  Validity + button_pressed, length)
 
vb <- rev(as.numeric(freq[1,5:7]/sum(freq[1,5:7])))
vu <- rev(as.numeric(freq[2,5:7]/sum(freq[1,5:7])))
 
ib <- rev(as.numeric(freq[1,2:4]/sum(freq[2,2:4])))
iu <- rev(as.numeric(freq[2,2:4]/sum(freq[2,2:4])))
 
CIvb <- multinomialCI(rev(as.numeric(freq[1,5:7])), 0.05)
CIvu <- multinomialCI(rev(as.numeric(freq[2,5:7])), 0.05)
 
CIib <- multinomialCI(rev(as.numeric(freq[1,2:4])), 0.05)
CIiu <- multinomialCI(rev(as.numeric(freq[2,2:4])), 0.05)


# set graphic constatnts
col_blue <- "#0028a5"
col_brown <- "#a57d00"

mgp1 <-  c(3.5, 3, 0)
mgp2 <-  c(3.5, 1.5, 0)
alwd <- 2
cex_main <- 5
main_line <- -3


# make a nice PDF file 

pdf("responses.pdf", height = 10.5, width = 20.5)
mat <- matrix(c(1,2,0,0),nrow = 2, ncol = 2, byrow = TRUE)
layout(mat, heights = c(0.9,0.1))

op <- par(cex.main = 5, mar = c(5, 10, 3, 2) + 0.1, mgp = c(3.5, 1, 0), cex.lab = 4,
          font.lab = 2, cex.axis = 4, bty = "n", las = 1, lwd = 4)
# valid
plot(NULL, ylim = c(0,1), xlim = c(0,2.5),ylab = "", xlab = "", main = "", axes = FALSE, cex = 4 )
title("valid", line = -1.5)

arrows(0.2,CIvb[1,1],0.2,CIvb[1,2], code=3, length=0.06, angle = 90)
arrows(1.2,CIvb[2,1],1.2,CIvb[2,2], code=3, length=0.06, angle = 90)
arrows(2.2,CIvb[3,1],2.2,CIvb[3,2], code=3, length=0.06, angle = 90)

arrows(0.4,CIvu[1,1],0.4,CIvu[1,2], code=3, length=0.06, angle = 90)
arrows(1.4,CIvu[2,1],1.4,CIvu[2,2], code=3, length=0.06, angle = 90)
arrows(2.4,CIvu[3,1],2.4,CIvu[3,2], code=3, length=0.06, angle = 90)

points(c(0.2,1.2,2.2,0.4,1.4,2.4),c(vb,vu),pch = 18, col = c(col_brown,col_brown,col_brown,col_blue,col_blue,col_blue), cex = 5.5)

axis(1, labels = c("Yes","I don't know","No"), at = c(0.3,1.3,2.3),mgp = mgp1, lwd = alwd)
axis(2, mgp = mgp2, lwd = alwd, las = 1)
par(las = 0)
mtext("Proportion", side = 2, line = 7, cex = 4)

legend(x = 0.8, y = 0.92, pch = 18, legend = c("believable","unbelievable"), col = c(col_brown,col_blue), bty = "n", cex = 5 ,pt.cex = 5.5)

# invalid 
plot(NULL, ylim = c(0,1), xlim = c(0,2.5),ylab = "", xlab = "", main = "", axes = FALSE, cex = 4 )
title("invalid", line = -1.5)

arrows(0.2,CIib[1,1],0.2,CIib[1,2], code=3, length=0.06, angle = 90)
arrows(1.2,CIib[2,1],1.2,CIib[2,2], code=3, length=0.06, angle = 90)
arrows(2.2,CIib[3,1],2.2,CIib[3,2], code=3, length=0.06, angle = 90)

arrows(0.4,CIiu[1,1],0.4,CIiu[1,2], code=3, length=0.06, angle = 90)
arrows(1.4,CIiu[2,1],1.4,CIiu[2,2], code=3, length=0.06, angle = 90)
arrows(2.4,CIiu[3,1],2.4,CIiu[3,2], code=3, length=0.06, angle = 90)

points(c(0.2,1.2,2.2,0.4,1.4,2.4),c(ib,iu),pch = 18, col = c(col_brown,col_brown,col_brown,col_blue,col_blue,col_blue), cex = 5.5)

axis(1, labels = c("Yes","I don't know","No"), at = c(0.3,1.3,2.3),mgp = mgp1, lwd = alwd)
axis(2, mgp = mgp2, lwd = alwd, las = 1)
par(las = 0)
mtext("Proportion", side = 2, line = 7, cex = 4)

title(main = "Response option", outer = TRUE, line = -62, cex.main = 5, font.main = 1)

dev.off()

#####################################################
######## EBB MPT with difference parameters #########
#####################################################

# we will run the model three times with differenr priors on the difference parameters

# priors
prior_1 = qnorm(0.7) # Narrow = 0.52
prior_2 = qnorm(0.8) # Medium = 0.84
prior_3 = qnorm(0.9) # Wide   = 1.28

# parts that all models have in common
nsubj <- nrow(dw)

inits <- function(npar = 8) {
  list(
    mu_ri_u=rnorm(1, 1), mu_rv_u=rnorm(1, 1), mu_ri_u=rnorm(1, 1), mu_rv_u=rnorm(1, 1), 
    mu_g_u=rnorm(1, 1), mu_g_u=rnorm(1, 1), mu_n_u=rnorm(1, 1), mu_n_u=rnorm(1, 1), 
    L_Omega=diag(npar), sigma = runif(npar),
    deltahat_tilde=matrix(rnorm(nsubj * npar, 0, 0.1), npar, nsubj)
  )
}

parameters <- c("mu_rib", "mu_riu", "mu_rvb", "mu_rvu",
                "mu_nb", "mu_nu", "mu_gb", "mu_gu",
                "delta_ri", "delta_rv", "delta_n", "delta_g",
                "Omega", "sigma", 
                "deltahat", 
                "lp__" , "T1")

ebbmpt <- stan_model(file = "extended_belief_bias_MPT.stan")

## Model 1: narrow prior ##

data_1 <- list(
  nsubj = nsubj,
  vbd = as.matrix(dw[,c( "believable_valid_yes", "believable_valid_unsure", "believable_valid_no")]),
  ibd = as.matrix(dw[,c( "believable_invalid_yes", "believable_invalid_unsure", "believable_invalid_no")]),
  vud = as.matrix(dw[,c( "unbelievable_valid_yes", "unbelievable_valid_unsure", "unbelievable_valid_no")]),
  iud = as.matrix(dw[,c( "unbelievable_invalid_yes", "unbelievable_invalid_unsure", "unbelievable_invalid_no")]),
  prior_sd_diff = prior_1
)

model_1 <- sampling(ebbmpt,
                    data=data_1, 
                    init=inits,
                    pars=parameters,
                    iter=2000, 
                    chains=6, 
                    thin=1,
                    warmup = 1000,
                    control = list(adapt_delta = 0.9, max_treedepth = 15)
)
tail(sort(summary(model_1)$summary[,"Rhat"]))
print(model_1, pars = parameters[1:12])

#stan_trace(model_1, pars = parameters[1:8])
#stan_trace(model_1, pars = parameters[9:16])

hyperpars_1 <- rstan::extract(model_1, pars = parameters[1:8])
deltas_1 <- rstan::extract(model_1, pars = parameters[9:12])
splines1 <- vapply(deltas_1, function(x) dlogspline(0, logspline(x)), 0)
dnorm(0, sd = prior_1)/splines1

#plot(density(deltas_1$delta_g))
#curve(dnorm(x, sd = prior_1), add = TRUE, lty = 2)

#save(model_1, file = "model_1.rda", compress = "xz")

## Model 2: medium prior ##

data_2 <- list(
  nsubj = nsubj,
  vbd = as.matrix(dw[,c( "believable_valid_yes", "believable_valid_unsure", "believable_valid_no")]),
  ibd = as.matrix(dw[,c( "believable_invalid_yes", "believable_invalid_unsure", "believable_invalid_no")]),
  vud = as.matrix(dw[,c( "unbelievable_valid_yes", "unbelievable_valid_unsure", "unbelievable_valid_no")]),
  iud = as.matrix(dw[,c( "unbelievable_invalid_yes", "unbelievable_invalid_unsure", "unbelievable_invalid_no")]),
  prior_sd_diff = prior_2
)

model_2 <- sampling(ebbmpt,
                    data=data_2, 
                    init=inits, 
                    pars=parameters,
                    iter=2000, 
                    chains=6, 
                    thin=1,
                    warmup = 1000,
                    control = list(adapt_delta = 0.9, max_treedepth = 15)
)
tail(sort(summary(model_2)$summary[,"Rhat"]))

#stan_trace(model_2, pars = parameters[1:8])
#stan_trace(model_2, pars = parameters[9:12])

hyperpars_2 <- rstan::extract(model_2, pars = parameters[1:8])
deltas_2 <- rstan::extract(model_2, pars = parameters[9:12])
splines2 <- vapply(deltas_2, function(x) dlogspline(0, logspline(x)), 0)
dnorm(0, sd = prior_2)/splines2

#plot(density(deltas_2$delta_g))
#curve(dnorm(x, sd = prior_2), add = TRUE, lty = 2)

#save(model_2, file = "model_2.rda", compress = "xz")

## Model 3: wide prior ##

data_3 <- list(
  nsubj = nsubj,
  vbd = as.matrix(dw[,c( "believable_valid_yes", "believable_valid_unsure", "believable_valid_no")]),
  ibd = as.matrix(dw[,c( "believable_invalid_yes", "believable_invalid_unsure", "believable_invalid_no")]),
  vud = as.matrix(dw[,c( "unbelievable_valid_yes", "unbelievable_valid_unsure", "unbelievable_valid_no")]),
  iud = as.matrix(dw[,c( "unbelievable_invalid_yes", "unbelievable_invalid_unsure", "unbelievable_invalid_no")]),
  prior_sd_diff = prior_3
)

model_3 <- sampling(ebbmpt,
                    data=data_3, 
                    init=inits, 
                    pars=parameters,
                    iter=2000, 
                    chains=6, 
                    thin=1,
                    warmup = 1000,
                    control = list(adapt_delta = 0.9, max_treedepth = 15)
)
tail(sort(summary(model_3)$summary[,"Rhat"]))

#stan_trace(model_3, pars = parameters[1:8])
#stan_trace(model_3, pars = parameters[9:12])

hyperpars_3 <- rstan::extract(model_3, pars = parameters[1:8])
deltas_3 <- rstan::extract(model_3, pars = parameters[9:12])
splines3 <- vapply(deltas_3, function(x) dlogspline(0, logspline(x)), 0)
dnorm(0, sd = prior_3)/splines3

plot(density(deltas_3$delta_g))
curve(dnorm(x, sd = prior_3), add = TRUE, lty = 2)

#save(model_3, file = "model_3.rda", compress = "xz")


## model fits (T1)

T1_model1 <- mean(unlist(rstan::extract(model_1, pars = "T1")))
T1_model2 <- mean(unlist(rstan::extract(model_2, pars = "T1")))
T1_model3 <- mean(unlist(rstan::extract(model_3, pars = "T1")))

## All the Bayes factors again

bf1 <- dnorm(0, sd = prior_1)/splines1
bf2 <- dnorm(0, sd = prior_2)/splines2
bf3 <- dnorm(0, sd = prior_3)/splines3

# BFs for the narrow prior
bf1
1/bf1
log(bf1)

# BFs for the medium prior
bf2
1/bf2
log(bf2)

# BFs for the wide prior
bf3
1/bf3
log(bf3)


# plot posterior estimates

col_blue <- "#0028a5"
col_brown <- "#a57d00"


mgp1 <-  c(3.5, 3, 0)
mgp2 <-  c(3.5, 1.5, 0)
alwd <- 2
cex_main <- 5
main_line <- -3


pdf("hyperparameters.pdf", height = 10.5, width = 20.5)

op <- par(cex.main = 6, mar = c(5,10, 3, 2) + 0.1, cex.lab = 4, 
          font.lab = 2, cex.axis = 4, bty = "n", las = 1, lwd = 7)

mat <- matrix(c(1,2,3,4,0,0),nrow = 3, ncol = 2, byrow = TRUE)
layout(mat, heights = c(0.46,0.46,0.08))

#par(mfrow= c(4,2))

#reasoning valid
plot(density(hyperpars_1$mu_rvb), xlim = c(0, 1), ylim = c(-0.01, 5), ylab = "", xlab = "", 
     main = "", axes = FALSE, family = "A")
lines(density(hyperpars_2$mu_rvb), col = col_blue)
lines(density(hyperpars_3$mu_rvb), col = col_brown)
mtext(text = expression(italic(r[v])), side = 3, line = main_line, xpd = NA, 
      cex = cex_main, at = c(0.2))

lines(density(hyperpars_1$mu_rvu), lty = 2)
lines(density(hyperpars_2$mu_rvu), lty = 2, col = col_blue)
lines(density(hyperpars_3$mu_rvu), lty = 2, col = col_brown)
axis(1, mgp = mgp1, lwd = alwd)
axis(2, mgp = mgp2, lwd = alwd)
par(las = 0)
mtext("Density", side = 2, line = 5, cex = 4)

#reasoning invalid
plot(density(hyperpars_1$mu_rib), xlim = c(0, 1), ylim = c(-0.01, 7), ylab = "", xlab = "", 
     main = "", axes = FALSE)
lines(density(hyperpars_2$mu_rib), col = col_blue)
lines(density(hyperpars_3$mu_rib), col = col_brown)
mtext(text = expression(italic(r[i])), side = 3, line = main_line, xpd = NA, 
      cex = cex_main, at = c(0.2))
lines(density(hyperpars_1$mu_riu),lty = 2)
lines(density(hyperpars_2$mu_riu),lty = 2, col = col_blue)
lines(density(hyperpars_3$mu_riu),lty = 2, col = col_brown)
axis(1, mgp = mgp1, lwd = alwd)
axis(2, mgp = mgp2, lwd = alwd)
par(las = 0)
legend("right", pch = 15, legend = c("narrow","medium","wide"), col = c("black",col_blue,col_brown), bty = "n", cex = 6)
mtext("Density", side = 2, line = 5, cex = 4)

# no answer
plot(density(hyperpars_1$mu_nb), xlim = c(0, 1), ylim = c(-0.01, 60), ylab = "", xlab = "", 
     main = "", axes = FALSE)
lines(density(hyperpars_2$mu_nb), col = col_blue)
lines(density(hyperpars_3$mu_nb), col = col_brown)
mtext(text = expression(italic(n)), side = 3, line = main_line, xpd = NA, 
      cex = cex_main, at = c(0.2))
lines(density(hyperpars_1$mu_nu), lty = 2)
lines(density(hyperpars_2$mu_nu), lty = 2, col = col_blue)
lines(density(hyperpars_3$mu_nu), lty = 2, col = col_brown)
axis(1, mgp = mgp1, lwd = alwd)
axis(2, mgp = mgp2, lwd = alwd)
par(las = 0)
legend("right", lty = c(1,2), legend = c("believable","unbelievable"), bty = "n", cex = 6)
mtext("Density", side = 2, line = 5, cex = 4)

# response bias
plot(density(hyperpars_1$mu_gb), xlim = c(0, 1), ylim = c(-0.01, 7), ylab = "", xlab = "", 
     main = "", axes = FALSE)
lines(density(hyperpars_2$mu_gb), col = col_blue)
lines(density(hyperpars_3$mu_gb), col = col_brown)
mtext(text = expression(italic(g)), side = 3, line = main_line, xpd = NA, 
      cex = cex_main, at = c(0.2))
lines(density(hyperpars_1$mu_gu), lty = 2)
lines(density(hyperpars_2$mu_gu), lty = 2, col = col_blue)
lines(density(hyperpars_3$mu_gu), lty = 2, col = col_brown)
axis(1, mgp = mgp1, lwd = alwd)
axis(2, mgp = mgp2, lwd = alwd)
par(las = 0)
mtext("Density", side = 2, line = 5, cex = 4)

title( "Parameter estimate", outer = TRUE, line = -79, cex.main = 6.5, font.main = 1)

dev.off()


